require File.dirname(__FILE__) + '/../test_helper'

class ContentViewerController
  append_view_path File.join(File.dirname(__FILE__) + '/../../views')
  def rescue_action(e)
    raise e
  end
end

class ContentViewerControllerTest < ActionController::TestCase

  def setup
    @profile = fast_create(Community)
    @track = CommunityTrackPlugin::Track.create!(:abstract => 'abstract', :body => 'body', :name => 'track', :profile => @profile)
    category = fast_create(Category, :name => "education")
    @track.add_category(category)

    @step = CommunityTrackPlugin::Step.create!(:name => 'step1', :body => 'body', :profile => @profile, :parent => @track, :published => false, :end_date => Date.today, :start_date => Date.today)

    user = create_user('testinguser')
    login_as(user.login)
    @profile.add_admin(user.person)
  end

  should 'show actions for tracks when user has permission for edit' do
    get :view_page, @track.url
    assert_tag :tag => 'div', :attributes => {:id => 'track' }, :descendant => { :tag => 'div', :attributes => { :class => 'actions' } }
  end

  should 'do not show actions for tracks when user has not permission for edit' do
    user = create_user('intruder')
    logout
    login_as(user.login)
    get :view_page, @track.url
    assert_no_tag :tag => 'div', :attributes => {:id => 'track' }, :descendant => { :tag => 'div', :attributes => { :class => 'actions' } }
  end

  should 'do not show new button at article toolbar for tracks' do
    user = create_user('intruder')
    logout
    login_as(user.login)
    get :view_page, @track.url
    assert_no_tag :tag => 'div', :attributes => {:id => 'article-actions'}, :descendant => { :tag => 'div', :attributes => { :id => 'icon-new' } }
  end

  should 'display steps for tracks' do
    get :view_page, @track.url
    assert_tag :tag => 'ul', :attributes => { :id => 'sortable' }, :descendant => {:tag => 'li', :attributes => { :class => 'step' } }
  end

  should 'display hidden field with step id' do
    get :view_page, @track.url
    assert_tag :tag => 'input', :attributes => { :name => 'step_ids[]' }
  end

  should 'show step' do
    get :view_page, @step.url
    assert_tag :tag => 'div', :attributes => { :id => 'step' }
  end

  should 'show tools for a step' do
    Article.create!(:profile => @profile, :name => 'article', :parent => @step)
    get :view_page, @step.url
    assert_tag :tag => 'div', :attributes => { :class => 'tools' }, :descendant => { :tag => 'div', :attributes => { :class => 'item' } }
  end

  should 'show actions for steps when user has permission for edit' do
    get :view_page, @step.url
    assert_tag :tag => 'div', :attributes => {:id => 'step' }, :descendant => { :tag => 'div', :attributes => { :class => 'actions' } }
  end

  should 'show actions for enabled tools in step' do
    get :view_page, @step.url
    assert_tag 'div', :attributes => {:class => 'actions' }, :descendant => { :tag => 'a', :attributes => { :class => 'button with-text icon-new icon-newforum' } }
    assert_tag 'div', :attributes => {:class => 'actions' }, :descendant => { :tag => 'a', :attributes => { :class => 'button with-text icon-new icon-newtext-html' } }
  end

  should 'do not show actions for steps when user has not permission for edit' do
    user = create_user('intruder')
    logout
    login_as(user.login)
    get :view_page, @step.url
    assert_no_tag :tag => 'div', :attributes => {:id => 'step' }, :descendant => { :tag => 'div', :attributes => { :class => 'actions' } }
  end

  should 'render a div with block id for track list block' do
    box = fast_create(Box, :owner_id => @profile.id, :owner_type => @profile.class.name)
    @block = CommunityTrackPlugin::TrackListBlock.create!(:box => box)
    @profile.boxes << box
    get :view_page, @step.url
    assert_tag :tag => 'div', :attributes => { :class => 'track_list', :id => "track_list_#{@block.id}" }
  end

  should 'render a div with block id for track card list block' do
    box = fast_create(Box, :owner_id => @profile.id, :owner_type => @profile.class.name)
    @block = CommunityTrackPlugin::TrackCardListBlock.create!(:box => box)
    @profile.boxes << box
    get :view_page, @step.url
    assert_tag :tag => 'div', :attributes => { :class => 'track_list', :id => "track_list_#{@block.id}" }
  end

  should 'render tracks in track list block' do
    box = fast_create(Box, :owner_id => @profile.id, :owner_type => @profile.class.name)
    @block = CommunityTrackPlugin::TrackListBlock.create!(:box => box)
    @profile.boxes << box
    get :view_page, @step.url
    assert_tag :tag => 'div', :attributes => { :class => 'item category_education' }, :descendant => { :tag => 'div', :attributes => { :class => 'steps' }, :descendant => { :tag => 'div', :attributes => { :class => "step #{@block.status_class(@step)}" } } }
  end

  should 'render tracks in track card list block' do
    box = fast_create(Box, :owner_id => @profile.id, :owner_type => @profile.class.name)
    @block = CommunityTrackPlugin::TrackCardListBlock.create!(:box => box)
    @profile.boxes << box
    get :view_page, @step.url
    assert_tag :tag => 'div', :attributes => { :class => 'item_card category_education' }, :descendant => { :tag => 'div', :attributes => { :class => 'track_content' } }
    assert_tag :tag => 'div', :attributes => { :class => 'item_card category_education' }, :descendant => { :tag => 'div', :attributes => { :class => 'track_stats' } }
  end

  should 'render link to display more tracks in track list block' do
    box = fast_create(Box, :owner_id => @profile.id, :owner_type => @profile.class.name)
    @block = CommunityTrackPlugin::TrackCardListBlock.create!(:box => box)
    @profile.boxes << box

    (@block.limit+1).times do |i|
      CommunityTrackPlugin::Track.create!(:abstract => 'abstract', :body => 'body', :name => "track#{i}", :profile => @profile)
    end

    get :view_page, @step.url
    assert_tag :tag => 'div', :attributes => { :id => "track_list_more_#{@block.id}" }, :descendant => { :tag => 'div', :attributes => { :class => 'more' } }
  end

  should 'render link to show all tracks in track list block' do
    box = fast_create(Box, :owner_id => @profile.id, :owner_type => @profile.class.name)
    @block = CommunityTrackPlugin::TrackCardListBlock.create!(:box => box)
    @profile.boxes << box
    @block.more_another_page = true
    @block.save!

    (@block.limit+1).times do |i|
      CommunityTrackPlugin::Track.create!(:abstract => 'abstract', :body => 'body', :name => "track#{i}", :profile => @profile)
    end

    get :view_page, @step.url
    assert_tag :tag => 'div', :attributes => { :id => "track_list_more_#{@block.id}" }, :descendant => { :tag => 'div', :attributes => { :class => 'view_all' } }
  end

end
